library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity Power is

    port (
		clk, start : in std_logic;
		base : in std_logic_vector(3 downto 0);
		exp  : in std_logic_vector(3 downto 0);
		done : out std_logic;
		d_out : out std_logic_vector(31 downto 0)
    );

end Power;

architecture std of Power is

	component Multiplier32b
    	port (
			a, b : in unsigned(31 downto 0);
			p : out unsigned(63 downto 0)
	    );
	end component;
	for mult : Multiplier32b use entity work.Multiplier32b;

	component SyncDownCounter
		port (
			clk, enable, load : in std_logic;
			d_in : in std_logic_vector(3 downto 0);
			d_out : out std_logic_vector(3 downto 0)
	    );
	end component;
	for counter : SyncDownCounter use entity work.SyncDownCounter;

	signal w_mult_a : std_logic_vector(31 downto 0);
	signal w_mult_b : std_logic_vector(31 downto 0);
	signal w_mult_p : unsigned(63 downto 0);
	signal w_mult_p_reg : std_logic_vector(31 downto 0);

	signal w_counter_enable : std_logic;
	signal w_counter_load : std_logic;
	signal w_counter_d_in : std_logic_vector(3 downto 0);
	signal w_counter_d_out : std_logic_vector(3 downto 0);

begin

	mult : Multiplier32b port map (unsigned(w_mult_a), unsigned(w_mult_b), w_mult_p);
	counter : SyncDownCounter port map (clk, w_counter_enable, w_counter_load, w_counter_d_in, w_counter_d_out);

	w_mult_a(31 downto 4) <= x"0000000";
	w_mult_a(3 downto 0) <= base;
	w_mult_b <= w_mult_p_reg;

	w_counter_enable <= start when (w_counter_d_out /= x"0") else '0';
	w_counter_load <= not start;
	w_counter_d_in <= exp;

	done <= '1' when (x"0" = w_counter_d_out) else '0';
	d_out <= w_mult_p_reg;

	process(clk)
		variable r_mult_p : std_logic_vector(63 downto 0);
	begin
		if(rising_edge(clk)) then
			if('0' = start) then
				r_mult_p := x"0000000000000001";
			elsif(w_counter_d_out /= x"0") then
				r_mult_p := std_logic_vector(w_mult_p);
			end if;

			w_mult_p_reg <= r_mult_p(31 downto 0);
		end if;
	end process;

end std;
